diff --git a/include/vlc/libvlc_media_player.h b/include/vlc/libvlc_media_player.h
index 8ac784a..c592028 100644
--- a/include/vlc/libvlc_media_player.h
+++ b/include/vlc/libvlc_media_player.h
@@ -1391,6 +1391,69 @@ VLC_PUBLIC_API int64_t libvlc_audio_get_delay( libvlc_media_player_t *p_mi );
  */
 VLC_PUBLIC_API int libvlc_audio_set_delay( libvlc_media_player_t *p_mi, int64_t i_delay );
 
+/**
+ * Set a random projectm preset
+ *
+ * \param p_mi media player
+ * \version LibVLC 1.1.6 or later
+ */
+VLC_PUBLIC_API void libvlc_projectm_random( libvlc_media_player_t *p_mi );
+
+/**
+ * Set a projectm preset
+ *
+ * \param p_mi media player
+ * \param i_id preset id
+ * \version LibVLC 1.1.6 or later
+ */
+VLC_PUBLIC_API void libvlc_projectm_set_preset( libvlc_media_player_t *p_mi, int i_id );
+
+/**
+ * Get projectm preset count
+ *
+ * \param p_mi media player
+ * \return number of presets loaded by projectm
+ * \version LibVLC 1.1.6 or later
+ */
+VLC_PUBLIC_API  int libvlc_projectm_get_preset_count( libvlc_media_player_t *p_mi );
+
+/**
+ * Get the current preset id
+ *
+ * \param p_mi media player
+ * \return current preset id
+ * \version LibVLC 1.1.6 or later
+ */
+VLC_PUBLIC_API  int libvlc_projectm_get_preset_current_id( libvlc_media_player_t *p_mi );
+
+/**
+ * Get the name of a preset
+ *
+ * \param p_mi media player
+ * \param i_id preset id
+ * \return name of the preset
+ * \version LibVLC 1.1.6 or later
+ */
+VLC_PUBLIC_API char *libvlc_projectm_get_preset_name( libvlc_media_player_t *p_mi, int i_id );
+
+/**
+ * Get whether projectm is locked or not
+ *
+ * \param p_mi media player
+ * \return the lock status (boolean)
+ * \version LibVLC 1.1.6 or later
+ */
+VLC_PUBLIC_API  int libvlc_projectm_is_locked( libvlc_media_player_t *p_mi );
+
+/**
+ * Lock/Unlock projectm on the current preset
+ *
+ * \param p_mi media player
+ * \param b_locked boolean for the lock status
+ * \version LibVLC 1.1.6 or later
+ */
+VLC_PUBLIC_API void libvlc_projectm_lock( libvlc_media_player_t *p_mi, int b_locked );
+
 /** @} audio */
 
 /** @} media_player */
diff --git a/include/vlc_aout.h b/include/vlc_aout.h
index 25c7e22..5e8503a 100644
--- a/include/vlc_aout.h
+++ b/include/vlc_aout.h
@@ -119,6 +119,15 @@ typedef int32_t vlc_fixed_t;
 #define AOUT_VAR_CHAN_RIGHT         4
 #define AOUT_VAR_CHAN_DOLBYS        5
 
+/* ProjectM stuff */
+#define VLC_PROJECTM_CMD_RANDOM      -1
+#define VLC_PROJECTM_CMD_LOCK        -2
+#define VLC_PROJECTM_CMD_UNLOCK      -3
+#define VLC_PROJECTM_CMD_IS_LOCKED   -4
+#define VLC_PROJECTM_CMD_GET_COUNT   -5
+#define VLC_PROJECTM_CMD_GET_CURRENT -6
+#define VLC_PROJECTM_CMD_GET_NAME    -7
+
 /*****************************************************************************
  * Main audio output structures
  *****************************************************************************/
diff --git a/modules/visualization/projectm.cpp b/modules/visualization/projectm.cpp
index 13e85d8..6c96576 100644
--- a/modules/visualization/projectm.cpp
+++ b/modules/visualization/projectm.cpp
@@ -64,6 +64,15 @@ static void Close        ( vlc_object_t * );
 #define HEIGHT_TEXT N_("Video height")
 #define HEIGHT_LONGTEXT N_("The height of the video window, in pixels.")
 
+#define MESHX_TEXT N_("Mesh width")
+#define MESHX_LONGTEXT N_("The width of the mesh, in pixels.")
+
+#define MESHY_TEXT N_("Mesh height")
+#define MESHY_LONGTEXT N_("The height of the mesh, in pixels.")
+
+#define TEXTURE_TEXT N_("Texture size")
+#define TEXTURE_LONGTEXT N_("The size of the texture, in pixels.")
+
 vlc_module_begin ()
     set_shortname( N_("projectM"))
     set_description( N_("libprojectM effect") )
@@ -85,6 +94,12 @@ vlc_module_begin ()
                  false )
     add_integer( "projectm-height", 640, HEIGHT_TEXT, HEIGHT_LONGTEXT,
                  false )
+    add_integer( "projectm-meshx", 32, MESHX_TEXT, MESHX_LONGTEXT,
+                 false )
+    add_integer( "projectm-meshy", 24, MESHY_TEXT, MESHY_LONGTEXT,
+                 false )
+    add_integer( "projectm-texture-size", 1024, TEXTURE_TEXT, TEXTURE_LONGTEXT,
+                 false )
     add_shortcut( "projectm" )
     set_callbacks( Open, Close )
 vlc_module_end ()
@@ -104,6 +119,9 @@ struct filter_sys_t
     vout_thread_t  *p_vout;
     vout_display_t *p_vd;
 
+    /* libprojectM objects */
+    projectM      *p_projectm;
+
     /* Window size */
     int i_width;
     int i_height;
@@ -117,6 +135,10 @@ struct filter_sys_t
     float *p_buffer;
     int   i_buffer_size;
     int   i_nb_samples;
+
+    /* projectm interaction */
+    bool b_changed;
+    int  i_preset_id;
 };
 
 
@@ -163,6 +185,7 @@ static int Open( vlc_object_t * p_this )
     p_sys->p_buffer = NULL;
     p_sys->i_buffer_size = 0;
     p_sys->i_nb_samples = 0;
+    p_sys->b_changed = false;
 
     /* Create the thread */
     if( vlc_clone( &p_sys->thread, Thread, p_filter, VLC_THREAD_PRIORITY_LOW ) )
@@ -257,6 +280,51 @@ static int VoutCallback( vlc_object_t *p_vout, char const *psz_name,
 }
 
 /**
+ * Variable callback for the preset change request
+ */
+static int ProjectMCallback( vlc_object_t *p_vout, char const *psz_name,
+                         vlc_value_t oldv, vlc_value_t newv, void *p_data )
+{
+    filter_t        *p_filter = (filter_t*)p_data;
+    filter_sys_t    *p_sys = p_filter->p_sys;
+
+    if( !strcmp(psz_name, "projectm-preset-cmd") )
+    {
+        if ( newv.i_int == VLC_PROJECTM_CMD_GET_COUNT )
+        {
+            var_SetInteger( p_filter, "projectm-preset-id", (int)p_sys->p_projectm->getPlaylistSize() );
+            return VLC_SUCCESS;
+        }
+        if ( newv.i_int == VLC_PROJECTM_CMD_GET_CURRENT )
+        {
+            unsigned int id;
+            p_sys->p_projectm->selectedPresetIndex( id );
+            var_SetInteger( p_filter, "projectm-preset-id", id );
+            return VLC_SUCCESS;
+        }
+        if ( newv.i_int == VLC_PROJECTM_CMD_GET_NAME )
+        {
+            int id = var_GetInteger( p_filter, "projectm-preset-id" );
+            if ( id >= (int)p_sys->p_projectm->getPlaylistSize() || id < 0 )
+                return VLC_EBADVAR;
+            var_SetString( p_filter, "projectm-preset-name",  p_sys->p_projectm->getPresetName( id ).c_str() );
+            return VLC_SUCCESS;
+        }
+        if ( newv.i_int == VLC_PROJECTM_CMD_IS_LOCKED )
+        {
+            var_SetInteger( p_filter, "projectm-preset-id", (int)p_sys->p_projectm->isPresetLocked() );
+            return VLC_SUCCESS;
+        }
+        if ( newv.i_int >= (int)p_sys->p_projectm->getPlaylistSize() || newv.i_int < VLC_PROJECTM_CMD_IS_LOCKED )
+            return VLC_EBADVAR;
+        // set_preset, random or lock
+        p_sys->i_preset_id = newv.i_int;
+        p_sys->b_changed = true;
+    }
+    return VLC_SUCCESS;
+}
+
+/**
  * ProjectM update thread which do the rendering
  * @param p_this: the p_thread object
  */
@@ -272,7 +340,6 @@ static void *Thread( void *p_data )
     locale_t loc;
     locale_t oldloc;
 
-    projectM *p_projectm;
 #ifndef HAVE_PROJECTM2
     char *psz_config;
 #else
@@ -330,17 +397,17 @@ static void *Thread( void *p_data )
     /* Create the projectM object */
 #ifndef HAVE_PROJECTM2
     psz_config = var_InheritString( p_filter, "projectm-config" );
-    p_projectm new projectM( psz_config );
+    p_sys->p_projectm = new projectM( psz_config );
     free( psz_config );
 #else
     psz_preset_path = var_InheritString( p_filter, "projectm-preset-path" );
     psz_title_font = var_InheritString( p_filter, "projectm-title-font" );
     psz_menu_font = var_InheritString( p_filter, "projectm-menu-font" );
 
-    settings.meshX = 32;
-    settings.meshY = 24;
+    settings.meshX = var_InheritInteger( p_filter, "projectm-meshx" );
+    settings.meshY = var_InheritInteger( p_filter, "projectm-meshy" );
     settings.fps = 35;
-    settings.textureSize = 1024;
+    settings.textureSize = var_InheritInteger( p_filter, "projectm-texture-size" );
     settings.windowWidth = p_sys->i_width;
     settings.windowHeight = p_sys->i_height;
     settings.presetURL = psz_preset_path;
@@ -352,25 +419,52 @@ static void *Thread( void *p_data )
     settings.aspectCorrection = 1;
     settings.easterEgg = 1;
     settings.shuffleEnabled = 1;
-    p_projectm = new projectM( settings );
+    p_sys->p_projectm = new projectM( settings );
 
     free( psz_menu_font );
     free( psz_title_font );
     free( psz_preset_path );
 #endif
-    p_sys->i_buffer_size = p_projectm->pcm()->maxsamples;
+    p_sys->i_buffer_size = p_sys->p_projectm->pcm()->maxsamples;
     p_sys->p_buffer = (float*)calloc( p_sys->i_buffer_size,
                                       sizeof( float ) );
 
     vlc_sem_post( &p_sys->ready );
 
     /* Choose a preset randomly or projectM will always show the first one */
-    p_projectm->selectPreset( (unsigned)vlc_mrand48() % p_projectm->getPlaylistSize() );
+    if ( p_sys->p_projectm->getPlaylistSize() > 0 )
+        p_sys->p_projectm->selectPreset( (unsigned)vlc_mrand48() % p_sys->p_projectm->getPlaylistSize() );
+
+    var_Create( p_filter, "projectm-preset-cmd", VLC_VAR_INTEGER );
+    var_Create( p_filter, "projectm-preset-id", VLC_VAR_INTEGER );
+    var_Create( p_filter, "projectm-preset-name", VLC_VAR_STRING );
+    var_AddCallback( p_filter, "projectm-preset-cmd", ProjectMCallback, p_filter );
 
     /* */
     for( ;; )
     {
         const mtime_t i_deadline = mdate() + CLOCK_FREQ / 50; /* 50 fps max */
+
+        /* Preset change requested */
+        if ( p_sys->b_changed )
+        {
+            switch ( p_sys->i_preset_id )
+            {
+                case VLC_PROJECTM_CMD_RANDOM:
+                  p_sys->p_projectm->key_handler( PROJECTM_KEYDOWN, PROJECTM_K_r, PROJECTM_KMOD_LSHIFT );
+                  break;
+                case VLC_PROJECTM_CMD_LOCK:
+                  p_sys->p_projectm->setPresetLock(TRUE);
+                  break;
+                case VLC_PROJECTM_CMD_UNLOCK:
+                  p_sys->p_projectm->setPresetLock(FALSE);
+                  break;
+                default: // set preset
+                  p_sys->p_projectm->selectPreset( p_sys->i_preset_id );
+            }
+            p_sys->b_changed = false;
+        }
+
         /* Manage the events */
         vout_ManageDisplay( p_sys->p_vd, true );
         if( p_sys->p_vd->cfg->display.width  != i_last_width ||
@@ -379,7 +473,7 @@ static void *Thread( void *p_data )
             /* FIXME it is not perfect as we will have black bands */
             vout_display_place_t place;
             vout_display_PlacePicture( &place, &p_sys->p_vd->source, p_sys->p_vd->cfg, false );
-            p_projectm->projectM_resetGL( place.width, place.height );
+            p_sys->p_projectm->projectM_resetGL( place.width, place.height );
 
             i_last_width  = p_sys->p_vd->cfg->display.width;
             i_last_height = p_sys->p_vd->cfg->display.height;
@@ -389,15 +483,15 @@ static void *Thread( void *p_data )
         vlc_mutex_lock( &p_sys->lock );
         if( p_sys->i_nb_samples > 0 )
         {
-            p_projectm->pcm()->addPCMfloat( p_sys->p_buffer,
-                                            p_sys->i_nb_samples );
+            p_sys->p_projectm->pcm()->addPCMfloat( p_sys->p_buffer,
+                                                   p_sys->i_nb_samples );
             p_sys->i_nb_samples = 0;
         }
         if( p_sys->b_quit )
         {
             vlc_mutex_unlock( &p_sys->lock );
 
-            delete p_projectm;
+            delete p_sys->p_projectm;
             vout_DeleteDisplay( p_sys->p_vd, NULL );
             vlc_object_release( p_sys->p_vout );
             if (loc != (locale_t)0)
@@ -409,7 +503,7 @@ static void *Thread( void *p_data )
         }
         vlc_mutex_unlock( &p_sys->lock );
 
-        p_projectm->renderFrame();
+        p_sys->p_projectm->renderFrame();
 
         /* */
         mwait( i_deadline );
diff --git a/src/audio_output/input.c b/src/audio_output/input.c
index 8fcd834..0870212 100644
--- a/src/audio_output/input.c
+++ b/src/audio_output/input.c
@@ -57,6 +57,8 @@ static void inputResamplingStop( aout_input_t *p_input );
 
 static int VisualizationCallback( vlc_object_t *, char const *,
                                   vlc_value_t, vlc_value_t, void * );
+static int ProjectmCallback( vlc_object_t *, char const *,
+                                  vlc_value_t, vlc_value_t, void * );
 static int EqualizerCallback( vlc_object_t *, char const *,
                               vlc_value_t, vlc_value_t, void * );
 static int ReplayGainCallback( vlc_object_t *, char const *,
@@ -373,6 +375,14 @@ int aout_InputNew( aout_instance_t * p_aout, aout_input_t * p_input, const aout_
 
             if( i_visual == 0 ) /* scaletempo */
                 p_input->p_playback_rate_filter = p_filter;
+           else if ( i_visual == 2 && !strcmp(psz_visual, "projectm") && /* projectm */
+                      var_Type( p_aout, "audio-projectm-preset-cmd" ) == 0 )
+            {
+                var_Create( p_aout, "audio-projectm-preset-cmd", VLC_VAR_INTEGER );
+                var_Create( p_aout, "audio-projectm-preset-id", VLC_VAR_INTEGER );
+                var_Create( p_aout, "audio-projectm-preset-name", VLC_VAR_STRING );
+                var_AddCallback( p_aout, "audio-projectm-preset-cmd", ProjectmCallback, p_filter );
+            }
 
             /* next filter if any */
             psz_parser = psz_next;
@@ -466,6 +476,10 @@ int aout_InputDelete( aout_instance_t * p_aout, aout_input_t * p_input )
     if ( p_input->b_error )
         return 0;
 
+    var_Destroy( p_aout, "audio-projectm-preset-cmd" );
+    var_Destroy( p_aout, "audio-projectm-preset-id" );
+    var_Destroy( p_aout, "audio-projectm-preset-name" );
+
     /* XXX We need to update b_recycle_vout before calling aout_FiltersDestroyPipeline.
      * FIXME They can be a race condition if audio-visual is updated between
      * aout_InputDelete and aout_InputNew.
@@ -791,6 +805,10 @@ static void inputFailure( aout_instance_t * p_aout, aout_input_t * p_input,
     var_Destroy( p_aout, "audio-replay-gain-preamp" );
     var_Destroy( p_aout, "audio-replay-gain-peak-protection" );
 
+    var_Destroy( p_aout, "audio-projectm-preset-cmd" );
+    var_Destroy( p_aout, "audio-projectm-preset-id" );
+    var_Destroy( p_aout, "audio-projectm-preset-name" );
+
     /* error flag */
     p_input->b_error = 1;
 }
@@ -895,6 +913,38 @@ static int VisualizationCallback( vlc_object_t *p_this, char const *psz_cmd,
     return VLC_SUCCESS;
 }
 
+static int ProjectmCallback( vlc_object_t *p_this, char const *psz_cmd,
+                             vlc_value_t oldval, vlc_value_t newval, void *p_data )
+{
+    aout_instance_t *p_aout = (aout_instance_t *)p_this;
+    filter_t *p_filter = (filter_t *)p_data;
+    int i_cmd = newval.i_int;
+    int i_id;
+    char *psz_name;
+
+    if ( i_cmd == VLC_PROJECTM_CMD_GET_NAME )
+    {
+        i_id = var_GetInteger( p_aout, "audio-projectm-preset-id" );
+        var_SetInteger( p_filter, "projectm-preset-id", i_id );
+        var_SetInteger( p_filter, "projectm-preset-cmd", i_cmd );
+        psz_name = var_GetString( p_filter, "projectm-preset-name" );
+        var_SetString( p_aout, "audio-projectm-preset-name",  psz_name );
+    }
+    else
+    {
+        var_SetInteger( p_filter, "projectm-preset-cmd", i_cmd );
+        if ( i_cmd == VLC_PROJECTM_CMD_GET_COUNT ||
+             i_cmd == VLC_PROJECTM_CMD_GET_CURRENT ||
+             i_cmd == VLC_PROJECTM_CMD_IS_LOCKED )
+        {
+            i_id = var_GetInteger( p_filter, "projectm-preset-id" );
+            var_SetInteger( p_aout, "audio-projectm-preset-id", i_id );
+        }
+    }
+
+    return VLC_SUCCESS;
+}
+
 static int EqualizerCallback( vlc_object_t *p_this, char const *psz_cmd,
                        vlc_value_t oldval, vlc_value_t newval, void *p_data )
 {
diff --git a/src/control/audio.c b/src/control/audio.c
index a6e7888..76c58bb 100644
--- a/src/control/audio.c
+++ b/src/control/audio.c
@@ -515,3 +515,94 @@ int libvlc_audio_set_delay( libvlc_media_player_t *p_mi, int64_t i_delay )
     }
     return ret;
 }
+
+/*****************************************************************************
+ * send_projectm_cmd : Find projectm, send a command, and return the object
+ *****************************************************************************/
+static int send_projectm_cmd( libvlc_media_player_t *p_mi, int i_cmd )
+{
+    int id = 0;
+    aout_instance_t *p_aout = GetAOut( p_mi );
+    if( !p_aout )
+        return id;
+
+    if( var_SetInteger( p_aout, "audio-projectm-preset-cmd", i_cmd ) < 0 )
+    {
+        libvlc_printerr( "audio-projectm-preset-cmd out of range" );
+        return id;
+    }
+    id = var_GetInteger( p_aout, "audio-projectm-preset-id" );
+    vlc_object_release( p_aout );
+    return id;
+}
+
+/*****************************************************************************
+ * libvlc_projectm_random : Set a random projectm preset
+ *****************************************************************************/
+void libvlc_projectm_random( libvlc_media_player_t *p_mi )
+{
+    send_projectm_cmd( p_mi, VLC_PROJECTM_CMD_RANDOM );
+}
+
+/*****************************************************************************
+ * libvlc_projectm_set_preset : Set a projectm preset
+ *****************************************************************************/
+void libvlc_projectm_set_preset( libvlc_media_player_t *p_mi, int i_id )
+{
+    send_projectm_cmd( p_mi, i_id );
+}
+
+/*****************************************************************************
+ * libvlc_projectm_get_preset_count : Get projectm preset count
+ *****************************************************************************/
+int libvlc_projectm_get_preset_count( libvlc_media_player_t *p_mi )
+{
+    return send_projectm_cmd( p_mi, VLC_PROJECTM_CMD_GET_COUNT );
+}
+
+/*****************************************************************************
+ * libvlc_projectm_get_preset_current_id : Get the current preset id
+ *****************************************************************************/
+int libvlc_projectm_get_preset_current_id( libvlc_media_player_t *p_mi )
+{
+    return send_projectm_cmd( p_mi, VLC_PROJECTM_CMD_GET_CURRENT );
+}
+
+/*****************************************************************************
+ * libvlc_projectm_get_preset_name : Get the name of a preset
+ *****************************************************************************/
+char *libvlc_projectm_get_preset_name( libvlc_media_player_t *p_mi, int i_id )
+{
+    char *name = NULL;
+    aout_instance_t *p_aout = GetAOut( p_mi );
+    if ( !p_aout )
+        return name;
+
+    if( var_SetInteger( p_aout, "audio-projectm-preset-id", i_id ) < 0 )
+    {
+        libvlc_printerr( "audio-projectm-preset-id out of range" );
+    }
+    else
+    {
+        var_SetInteger( p_aout, "audio-projectm-preset-cmd", VLC_PROJECTM_CMD_GET_NAME );
+        name = var_GetString( p_aout, "audio-projectm-preset-name" );
+    }
+    vlc_object_release( p_aout );
+    return strdup(name);
+}
+
+/*****************************************************************************
+ * libvlc_projectm_is_locked : Get whether projectm is locked or not
+ *****************************************************************************/
+int libvlc_projectm_is_locked( libvlc_media_player_t *p_mi )
+{
+    return send_projectm_cmd( p_mi, VLC_PROJECTM_CMD_IS_LOCKED );
+}
+
+/*****************************************************************************
+ * libvlc_projectm_lock : Lock/Unlock projectm on the current preset
+ *****************************************************************************/
+void libvlc_projectm_lock( libvlc_media_player_t *p_mi, int b_locked )
+{
+    send_projectm_cmd( p_mi, b_locked ? VLC_PROJECTM_CMD_LOCK : VLC_PROJECTM_CMD_UNLOCK );
+}
diff --git a/src/libvlc.sym b/src/libvlc.sym
index cdd34ff..50af7b0 100644
--- a/src/libvlc.sym
+++ b/src/libvlc.sym
@@ -162,6 +162,13 @@ libvlc_media_subitems
 libvlc_new
 libvlc_new_with_builtins
 libvlc_playlist_play
+libvlc_projectm_get_preset_count
+libvlc_projectm_get_preset_current_id
+libvlc_projectm_get_preset_name
+libvlc_projectm_set_preset
+libvlc_projectm_random
+libvlc_projectm_is_locked
+libvlc_projectm_lock
 libvlc_release
 libvlc_retain
 libvlc_set_fullscreen
